import os
from typing import Annotated, Self
import logging

from pydantic import BaseModel, Field, model_validator
from pydantic_yaml import parse_yaml_file_as
from fastapi import Depends

from xmpp_api.exceptions.base import ConfigurationException


log = logging.getLogger("config")


class _ComponentConfig(BaseModel):
    # The JID of the component
    jid: str

    # Address of server's component port
    server: str

    # The component's secret.
    secret: str | None = Field(default=None)

    # Optional file to read the secret from
    secret_file: str | None = Field(default=None)

    @model_validator(mode="after")
    def validate_secret(self) -> Self:
        if self.secret is None and self.secret_file is None:
            raise ConfigurationException("No component secret is specified")
        if self.secret is not None and self.secret_file is not None:
            log.warn(
                "Both secret and secret_file specified! secret_file takes precedence"
            )
        return self


class _Config(BaseModel):
    # DB URI for sqlmodel
    database: str

    # Component configuration
    component: _ComponentConfig


def load_config() -> _Config:
    """
    Load the application config
    """

    config_path = "/etc/xmpp-api/config.yaml"
    if (env_config_file := os.environ.get("XMPP_API__CONFIG_FILE")) is not None:
        config_path = env_config_file

    log.info(f"Reading configuration from {config_path}")
    with open(config_path, "r", encoding="utf8") as f:
        config = parse_yaml_file_as(_Config, f)

    # Read the secret from a file, if specified
    if config.component.secret_file is not None:
        log.info("Reading component secret from %s", config.component.secret_file)
        with open(config.component.secret_file, "r", encoding="utf8") as f:
            config.component.secret = f.read().strip().replace("\n", "")

    return config


ConfigDep = Annotated[_Config, Depends(load_config)]
